'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _deleteProperty = require('babel-runtime/core-js/reflect/delete-property');

var _deleteProperty2 = _interopRequireDefault(_deleteProperty);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _assign2 = require('babel-runtime/core-js/object/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _pick2 = require('lodash/pick');

var _pick3 = _interopRequireDefault(_pick2);

var _chunk2 = require('lodash/chunk');

var _chunk3 = _interopRequireDefault(_chunk2);

var _defaults2 = require('lodash/defaults');

var _defaults3 = _interopRequireDefault(_defaults2);

var _assign4 = require('lodash/assign');

var _assign5 = _interopRequireDefault(_assign4);

var _sparkCore = require('@ciscospark/spark-core');

var _realtime = require('./realtime');

var _realtime2 = _interopRequireDefault(_realtime);

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _es6PromiseSeries = require('es6-promise-series');

var _es6PromiseSeries2 = _interopRequireDefault(_es6PromiseSeries);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*!
 * Copyright (c) 2015-2017 Cisco Systems, Inc. See LICENSE file.
 */

var Board = _sparkCore.SparkPlugin.extend({
  namespace: 'Board',

  children: {
    realtime: _realtime2.default
  },

  /**
   * Adds Content to a Channel
   * If contents length is greater than config.board.numberContentsPerPageForAdd, this method
   * will break contents into chunks and make multiple GET request to the
   * board service
   * @memberof Board.BoardService
   * @param  {Board~Channel} channel
   * @param  {Array} contents - Array of {@link Board~Content} objects
   * @returns {Promise<Board~Content>}
   */
  addContent: function addContent(channel, contents) {
    var _this = this;

    var chunks = [];

    chunks = (0, _chunk3.default)(contents, this.config.numberContentsPerPageForAdd);

    // we want the first promise to resolve before continuing with the next
    // chunk or else we'll have race conditions among patches
    return (0, _es6PromiseSeries2.default)(chunks.map(function (part) {
      return _this._addContentChunk.bind(_this, channel, part);
    }));
  },


  /**
   * Adds Image to a Channel
   * Uploads image to spark files and adds SCR + downloadUrl to the persistence
   * service
   * @memberof Board.BoardService
   * @param  {Board~Channel} channel
   * @param  {File} image - image to be uploaded
   * @param  {Object} metadata - metadata such as displayName
   * @returns {Promise<Board~Content>}
   */
  addImage: function addImage(channel, image, metadata) {
    var _this2 = this;

    return this.spark.internal.board._uploadImage(channel, image).then(function (scr) {
      return _this2.spark.internal.board.addContent(channel, [{
        type: 'FILE',
        metadata: metadata,
        file: {
          mimeType: image.type,
          scr: scr,
          size: image.size,
          url: scr.loc
        }
      }]);
    });
  },


  /**
   * Set a snapshot image for a board
   *
   * @param {Board~Channel} channel
   * @param {File} image
   * @returns {Promise<Board~Channel>}
   */
  setSnapshotImage: function setSnapshotImage(channel, image) {
    var _this3 = this;

    var imageScr = void 0;

    return this.spark.internal.board._uploadImage(channel, image, { hiddenSpace: true }).then(function (scr) {
      imageScr = scr;

      return _this3.spark.internal.encryption.encryptScr(channel.defaultEncryptionKeyUrl, imageScr);
    }).then(function (encryptedScr) {
      imageScr.encryptedScr = encryptedScr;

      return encryptedScr;
    }).then(function () {
      var imageBody = {
        image: {
          url: imageScr.loc,
          height: image.height || 900,
          width: image.width || 1600,
          mimeType: image.type || 'image/png',
          scr: imageScr.encryptedScr,
          encryptionKeyUrl: channel.defaultEncryptionKeyUrl,
          fileSize: image.size
        }
      };

      return _this3.spark.request({
        method: 'PATCH',
        api: 'board',
        resource: '/channels/' + channel.channelId,
        body: imageBody
      });
    }).then(function (res) {
      return res.body;
    });
  },


  /**
   * Creates a Channel
   * @memberof Board.BoardService
   * @param  {Conversation~ConversationObject} conversation
   * @param  {Board~Channel} channel
   * @returns {Promise<Board~Channel>}
   */
  createChannel: function createChannel(conversation, channel) {
    return this.spark.request({
      method: 'POST',
      api: 'board',
      resource: '/channels',
      body: this._prepareChannel(conversation, channel)
    }).then(function (res) {
      return res.body;
    });
  },
  _prepareChannel: function _prepareChannel(conversation, channel) {
    return (0, _assign3.default)({
      aclUrlLink: conversation.aclUrl,
      kmsMessage: {
        method: 'create',
        uri: '/resources',
        userIds: [conversation.kmsResourceObjectUrl],
        keyUris: []
      }
    }, channel);
  },


  /**
   * Deletes a Channel from a Conversation
   * @memberof Board.BoardService
   * @param  {Conversation~ConversationObject} conversation
   * @param  {Board~Channel} channel
   * @param  {Object} options
   * @param  {Object} options.preventDeleteActiveChannel Returns error if channel is in use
   * @returns {Promise}
   */
  deleteChannel: function deleteChannel(conversation, channel) {
    var _this4 = this;

    var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    // remove the ACL link between conversation and board
    // remove conversation auth from board KRO in kms message
    var body = {
      aclLinkType: 'INCOMING',
      linkedAcl: conversation.aclUrl,
      kmsMessage: {
        method: 'delete',
        uri: channel.kmsResourceUrl + '/authorizations?' + _querystring2.default.stringify({ authId: conversation.kmsResourceObjectUrl })
      },
      aclLinkOperation: 'DELETE'
    };

    var promise = _promise2.default.resolve();

    if (options.preventDeleteActiveChannel) {
      promise = this.lockChannelForDeletion(channel);
    }

    return promise.then(function () {
      return _this4.spark.request({
        method: 'PUT',
        uri: channel.aclUrl + '/links',
        body: body
      });
    }).then(function (res) {
      return res.body;
    });
  },


  /**
   * Locks and marks a channel for deletion
   * If a channel is being used, it will return 409 - Conflict
   * @memberof Board.BoardService
   * @param  {Board~Channel} channel
   * @returns {Promise}
   */
  lockChannelForDeletion: function lockChannelForDeletion(channel) {
    return this.spark.request({
      method: 'POST',
      uri: channel.channelUrl + '/lock',
      qs: {
        intent: 'delete'
      }
    }).then(function (res) {
      return res.body;
    });
  },


  /**
   * Keeps a channel as 'active' to prevent other people from deleting it
   * @param  {Board~Channel} channel
   * @returns {Promise}
   */
  keepActive: function keepActive(channel) {
    return this.spark.request({
      method: 'POST',
      uri: channel.channelUrl + '/keepAlive'
    });
  },


  /**
   * Decrypts a collection of content objects
   *
   * @memberof Board.BoardService
   * @param  {Array} contents curves, text, and images
   * @returns {Promise<Array>} Resolves with an array of {@link Board~Content} objects.
   */
  decryptContents: function decryptContents(contents) {
    var _this5 = this;

    return _promise2.default.all(contents.items.map(function (content) {
      var decryptPromise = void 0;

      if (content.type === 'FILE') {
        decryptPromise = _this5.decryptSingleFileContent(content.encryptionKeyUrl, content);
      } else {
        decryptPromise = _this5.decryptSingleContent(content.encryptionKeyUrl, content.payload);
      }

      return decryptPromise.then(function (res) {
        (0, _deleteProperty2.default)(content, 'payload');
        (0, _deleteProperty2.default)(content, 'encryptionKeyUrl');

        return (0, _defaults3.default)(res, content);
      });
    }));
  },


  /**
   * Decryts a single STRING content object
   * @memberof Board.BoardService
   * @param  {string} encryptionKeyUrl
   * @param  {string} encryptedData
   * @returns {Promise<Board~Content>}
   */
  decryptSingleContent: function decryptSingleContent(encryptionKeyUrl, encryptedData) {
    return this.spark.internal.encryption.decryptText(encryptionKeyUrl, encryptedData).then(function (res) {
      return JSON.parse(res);
    });
  },


  /**
   * Decryts a single FILE content object
   * @memberof Board.BoardService
   * @param  {string} encryptionKeyUrl
   * @param  {object} encryptedContent {file, payload}
   * @returns {Promise<Board~Content>}
   */
  decryptSingleFileContent: function decryptSingleFileContent(encryptionKeyUrl, encryptedContent) {
    var _this6 = this;

    var metadata = void 0;

    if (encryptedContent.payload) {
      metadata = encryptedContent.payload;
    }

    return this.spark.internal.encryption.decryptScr(encryptionKeyUrl, encryptedContent.file.scr).then(function (scr) {
      encryptedContent.file.scr = scr;
      if (metadata) {
        return _this6.spark.internal.encryption.decryptText(encryptionKeyUrl, metadata);
      }

      return '';
    }).then(function (decryptedMetadata) {
      try {
        encryptedContent.metadata = JSON.parse(decryptedMetadata);
        if (encryptedContent.metadata.displayName) {
          encryptedContent.displayName = encryptedContent.metadata.displayName;
        }
      } catch (error) {
        encryptedContent.metadata = {};
      }

      return encryptedContent;
    });
  },


  /**
   * Deletes all Content from a Channel
   * @memberof Board.BoardService
   * @param  {Board~Channel} channel
   * @returns {Promise} Resolves with an content response
   */
  deleteAllContent: function deleteAllContent(channel) {
    return this.spark.request({
      method: 'DELETE',
      uri: channel.channelUrl + '/contents'
    }).then(function (res) {
      return res.body;
    });
  },


  /**
   * Deletes Contents from a Channel except the ones listed in contentsToKeep
   * @memberof Board.BoardService
   * @param  {Board~Channel} channel
   * @param  {Array<Board~Content>} contentsToKeep Array of board objects (curves, text, and images) with valid contentId (received from server)
   * @returns {Promise} Resolves with an content response
   */
  deletePartialContent: function deletePartialContent(channel, contentsToKeep) {
    var body = contentsToKeep.map(function (content) {
      return (0, _pick3.default)(content, 'contentId');
    });

    return this.spark.request({
      method: 'POST',
      uri: channel.channelUrl + '/contents',
      body: body,
      qs: {
        clearBoard: true
      }
    }).then(function (res) {
      return res.body;
    });
  },


  /**
   * Encrypts a collection of content
   * @memberof Board.BoardService
   * @param  {string} encryptionKeyUrl channel.defaultEncryptionKeyUrl
   * @param  {Array} contents   Array of {@link Board~Content} objects. (curves, text, and images)
   * @returns {Promise<Array>} Resolves with an array of encrypted {@link Board~Content} objects.
   */
  encryptContents: function encryptContents(encryptionKeyUrl, contents) {
    var _this7 = this;

    return _promise2.default.all(contents.map(function (content) {
      var encryptionPromise = void 0;
      var contentType = 'STRING';

      // the existence of an scr will determine if the content is a FILE.
      if (content.file) {
        contentType = 'FILE';
        encryptionPromise = _this7.encryptSingleFileContent(encryptionKeyUrl, content);
      } else {
        encryptionPromise = _this7.encryptSingleContent(encryptionKeyUrl, content);
      }

      return encryptionPromise.then(function (res) {
        return (0, _assign5.default)({
          device: _this7.spark.internal.device.deviceType,
          type: contentType,
          encryptionKeyUrl: encryptionKeyUrl
        }, (0, _pick3.default)(res, 'file', 'payload'));
      });
    }));
  },


  /**
   * Encrypts a single STRING content object
   * @memberof Board.BoardService
   * @param  {string} encryptionKeyUrl
   * @param  {Board~Content} content
   * @returns {Promise<Board~Content>}
   */
  encryptSingleContent: function encryptSingleContent(encryptionKeyUrl, content) {
    return this.spark.internal.encryption.encryptText(encryptionKeyUrl, (0, _stringify2.default)(content)).then(function (res) {
      return {
        payload: res,
        encryptionKeyUrl: encryptionKeyUrl
      };
    });
  },


  /**
   * Encrypts a single FILE content object
   * @memberof Board.BoardService
   * @param  {string} encryptionKeyUrl
   * @param  {Board~Content} content
   * @returns {Promise<Board~Content>}
   */
  encryptSingleFileContent: function encryptSingleFileContent(encryptionKeyUrl, content) {
    var _this8 = this;

    return this.spark.internal.encryption.encryptScr(encryptionKeyUrl, content.file.scr).then(function (encryptedScr) {
      content.file.scr = encryptedScr;
      if (content.displayName) {
        content.metadata = (0, _assign5.default)(content.metadata, { displayName: content.displayName });
      }
      if (content.metadata) {
        return _this8.spark.internal.encryption.encryptText(encryptionKeyUrl, (0, _stringify2.default)(content.metadata)).then(function (encryptedMetadata) {
          content.metadata = encryptedMetadata;
        });
      }

      return content;
    }).then(function () {
      return {
        file: content.file,
        payload: content.metadata,
        encryptionKeyUrl: encryptionKeyUrl
      };
    });
  },


  /**
   * Retrieves contents from a specified channel
   * @memberof Board.BoardService
   * @param  {Board~Channel} channel
   * @param  {Object} options
   * @param  {Object} options.qs
   * @returns {Promise<Page<Board~Channel>>} Resolves with an array of Content items
   */
  getContents: function getContents(channel, options) {
    var _this9 = this;

    options = options || {};

    var params = {
      uri: channel.channelUrl + '/contents',
      qs: {
        contentsLimit: this.config.numberContentsPerPageForGet
      }
    };

    (0, _assign5.default)(params.qs, (0, _pick3.default)(options, 'contentsLimit'));

    return this.request(params).then(function (res) {
      return new _sparkCore.Page(res, _this9.spark);
    });
  },


  /**
   * Gets a Channel
   * @memberof Board.BoardService
   * @param  {Board~Channel} channel
   * @returns {Promise<Board~Channel>}
   */
  getChannel: function getChannel(channel) {
    return this.spark.request({
      method: 'GET',
      uri: channel.channelUrl
    }).then(function (res) {
      return res.body;
    });
  },


  /**
   * Gets Channels
   * @memberof Board.BoardService
   * @param {Conversation~ConversationObject} conversation
   * @param {Object} options
   * @param {number} options.channelsLimit number of boards to return per page
   * @param {number} options.type type of whiteboard: whiteboard or annotated
   * @returns {Promise<Page<Board~Channel>>} Resolves with an array of Channel items
   */
  getChannels: function getChannels(conversation, options) {
    var _this10 = this;

    options = options || {};

    if (!conversation) {
      return _promise2.default.reject(new Error('`conversation` is required'));
    }

    var params = {
      api: 'board',
      resource: '/channels',
      qs: {
        aclUrlLink: conversation.aclUrl
      }
    };

    (0, _assign5.default)(params.qs, (0, _pick3.default)(options, 'channelsLimit', 'type'));

    return this.request(params).then(function (res) {
      return new _sparkCore.Page(res, _this10.spark);
    });
  },


  /**
   * Pings persistence
   * @memberof Board.BoardService
   * @returns {Promise<Object>} ping response body
   */
  ping: function ping() {
    return this.spark.request({
      method: 'GET',
      api: 'board',
      resource: '/ping'
    }).then(function (res) {
      return res.body;
    });
  },
  processActivityEvent: function processActivityEvent(message) {
    var decryptionPromise = void 0;

    if (message.contentType === 'FILE') {
      decryptionPromise = this.decryptSingleFileContent(message.envelope.encryptionKeyUrl, message.payload);
    } else {
      decryptionPromise = this.decryptSingleContent(message.envelope.encryptionKeyUrl, message.payload);
    }

    return decryptionPromise.then(function (decryptedData) {
      // call the event handlers
      message.payload = decryptedData;

      return message;
    });
  },


  /**
   * Registers with Mercury
   * @memberof Board.BoardService
   * @param  {Object} data - Mercury bindings
   * @returns {Promise<Board~Registration>}
   */
  register: function register(data) {
    return this.spark.request({
      method: 'POST',
      api: 'board',
      resource: '/registrations',
      body: data
    }).then(function (res) {
      return res.body;
    });
  },


  /**
   * Registers with Mercury for sharing web socket
   * @memberof Board.BoardService
   * @param  {Board~Channel} channel
   * @returns {Promise<Board~Registration>}
   */
  registerToShareMercury: function registerToShareMercury(channel) {
    var _this11 = this;

    return this.spark.internal.feature.getFeature('developer', 'web-shared-mercury').then(function (isSharingMercuryFeatureEnabled) {
      if (!_this11.spark.internal.mercury.localClusterServiceUrls) {
        return _promise2.default.reject(new Error('`localClusterServiceUrls` is not defined, make sure mercury is connected'));
      }
      if (!isSharingMercuryFeatureEnabled) {
        return _promise2.default.reject(new Error('`web-shared-mercury` is not enabled'));
      }

      var webSocketUrl = _this11.spark.internal.device.webSocketUrl;
      var mercuryConnectionServiceClusterUrl = _this11.spark.internal.mercury.localClusterServiceUrls.mercuryConnectionServiceClusterUrl;


      var data = {
        mercuryConnectionServiceClusterUrl: mercuryConnectionServiceClusterUrl,
        webSocketUrl: webSocketUrl,
        action: 'ADD'
      };

      return _this11.spark.request({
        method: 'POST',
        uri: channel.channelUrl + '/register',
        body: data
      });
    }).then(function (res) {
      return res.body;
    });
  },


  /**
   * Remove board binding from existing mercury connection
   * @memberof Board.BoardService
   * @param  {Board~Channel} channel
   * @param  {String} binding - the binding as provided in board registration
   * @returns {Promise<Board~Registration>}
   */
  unregisterFromSharedMercury: function unregisterFromSharedMercury(channel, binding) {
    var webSocketUrl = this.spark.internal.device.webSocketUrl;

    var data = {
      binding: binding,
      webSocketUrl: webSocketUrl,
      action: 'REMOVE'
    };

    return this.spark.request({
      method: 'POST',
      uri: channel.channelUrl + '/register',
      body: data
    }).then(function (res) {
      return res.body;
    });
  },
  _addContentChunk: function _addContentChunk(channel, contentChunk) {
    var _this12 = this;

    return this.spark.internal.board.encryptContents(channel.defaultEncryptionKeyUrl, contentChunk).then(function (res) {
      return _this12.spark.request({
        method: 'POST',
        uri: channel.channelUrl + '/contents',
        body: res
      });
    }).then(function (res) {
      return res.body;
    });
  },


  /**
   * Encrypts and uploads image to SparkFiles
   * @memberof Board.BoardService
   * @param  {Board~Channel} channel
   * @param  {File} file - File to be uploaded
   * @param  {Object} options
   * @param  {Object} options.hiddenSpace - true for hidden, false for open space
   * @private
   * @returns {Object} Encrypted Scr and KeyUrl
   */
  _uploadImage: function _uploadImage(channel, file, options) {
    var _this13 = this;

    options = options || {};

    return this.spark.internal.encryption.encryptBinary(file).then(function (_ref) {
      var scr = _ref.scr,
          cdata = _ref.cdata;
      return _promise2.default.all([scr, _this13._uploadImageToSparkFiles(channel, cdata, options.hiddenSpace)]);
    }).then(function (_ref2) {
      var _ref3 = (0, _slicedToArray3.default)(_ref2, 2),
          scr = _ref3[0],
          res = _ref3[1];

      return (0, _assign5.default)(scr, { loc: res.downloadUrl });
    });
  },
  _getSpaceUrl: function _getSpaceUrl(channel, hiddenSpace) {
    var requestUri = channel.channelUrl + '/spaces/open';

    if (hiddenSpace) {
      requestUri = channel.channelUrl + '/spaces/hidden';
    }

    return this.spark.request({
      method: 'PUT',
      uri: requestUri
    }).then(function (res) {
      return res.body.spaceUrl;
    });
  },
  _uploadImageToSparkFiles: function _uploadImageToSparkFiles(channel, file, hiddenSpace) {
    var _this14 = this;

    var fileSize = file.length || file.size || file.byteLength;

    return this._getSpaceUrl(channel, hiddenSpace).then(function (spaceUrl) {
      return _this14.spark.upload({
        uri: spaceUrl + '/upload_sessions',
        file: file,
        qs: {
          transcode: true
        },
        phases: {
          initialize: { fileSize: fileSize },
          upload: {
            $url: function $url(session) {
              return session.uploadUrl;
            }
          },
          finalize: {
            $uri: function $uri(session) {
              return session.finishUploadUrl;
            },

            body: { fileSize: fileSize }
          }
        }
      });
    });
  },
  version: '1.52.7'
});

exports.default = Board;
//# sourceMappingURL=board.js.map
