'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

var _common = require('@ciscospark/common');

var _sparkCore = require('@ciscospark/spark-core');

var _realtimeChannelCollection = require('./realtime-channel-collection.js');

var _realtimeChannelCollection2 = _interopRequireDefault(_realtimeChannelCollection);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @class
 * @extends {Mercury}
 * @memberof Board
 */
/*!
 * Copyright (c) 2015-2017 Cisco Systems, Inc. See LICENSE file.
 */

var RealtimeService = _sparkCore.SparkPlugin.extend({
  namespace: 'Board',

  collections: {
    realtimeChannels: _realtimeChannelCollection2.default
  },

  /**
    * Sends the message via the socket. Assumes that the message is already properly formatted
    * @memberof Board.RealtimeService
    * @param {Board~Channel} channel
    * @param {string} message   Contains the un-encrypted message to send.
    * @returns {Promise<Board~Content>}
    */
  publish: function publish(channel, message) {
    var _this = this;

    var encryptionPromise = void 0;
    var contentType = 'STRING';

    if (message.payload.file) {
      contentType = 'FILE';
      encryptionPromise = this.spark.internal.board.encryptSingleFileContent(channel.defaultEncryptionKeyUrl, message.payload);
    } else {
      encryptionPromise = this.spark.internal.board.encryptSingleContent(channel.defaultEncryptionKeyUrl, message.payload);
    }

    return encryptionPromise.then(function (encryptedPayloadAndKeyUrl) {
      return _this.publishEncrypted(channel, encryptedPayloadAndKeyUrl, contentType);
    });
  },


  /**
    * Sends the message via the socket. The message should already have been
    * encrypted
    * @memberof Board.RealtimeService
    * @param {Board~Channel} channel
    * @param {object} encryptedPayloadAndKeyUrl
    * @param {string} contentType - provides hint for decryption. Defaults to
    * `STRING`, and could also be `FILE`
    * @returns {Promise<Board~Content>}
    */
  publishEncrypted: function publishEncrypted(channel, encryptedPayloadAndKeyUrl, contentType) {
    var realtimeChannel = this.realtimeChannels.get(channel.channelId);

    if (!realtimeChannel) {
      return _promise2.default.reject(new Error('Realtime Channel not found'));
    }

    var data = {
      id: _uuid2.default.v4(),
      type: 'publishRequest',
      recipients: [{
        alertType: 'none',
        route: realtimeChannel.binding,
        headers: {}
      }],
      data: {
        eventType: 'board.activity',
        contentType: contentType,
        payload: encryptedPayloadAndKeyUrl.payload,
        envelope: {
          encryptionKeyUrl: encryptedPayloadAndKeyUrl.encryptionKeyUrl,
          channelId: channel.channelId
        }
      }
    };

    // provide a hint for decryption
    if (contentType === 'FILE') {
      data.data.payload = {
        file: encryptedPayloadAndKeyUrl.file,
        payload: encryptedPayloadAndKeyUrl.payload
      };
    }

    return realtimeChannel.send(data);
  },
  createRealtimeChannel: function createRealtimeChannel(channel) {
    var _this2 = this;

    var requestBindings = [this._boardChannelIdToMercuryBinding(channel.channelId)];
    var bindingObj = { bindings: requestBindings };

    return this.spark.internal.board.register(bindingObj).then(function (_ref) {
      var webSocketUrl = _ref.webSocketUrl,
          bindings = _ref.bindings;

      _this2.realtimeChannels.add({
        channelId: channel.channelId,
        socketUrl: webSocketUrl,
        binding: bindings[0]
      });

      return _this2.realtimeChannels.get(channel.channelId);
    });
  },


  /**
    * Open new mercury connection
    * @memberof Board.RealtimeService
    * @param   {Board~Channel} channel
    * @returns {Promise}
    */
  connectByOpenNewMercuryConnection: function connectByOpenNewMercuryConnection(channel) {
    var _this3 = this;

    var promise = _promise2.default.resolve();
    var realtimeChannel = this.realtimeChannels.get(channel.channelId);

    if (!realtimeChannel) {
      this.logger.info('board realtime: realtime channel not found, creating new channel');
      promise = this.createRealtimeChannel(channel).then(function (rc) {
        realtimeChannel = rc;
        _this3.logger.info('board realtime: realtime channel created');

        return realtimeChannel;
      });
    }

    return promise.then(function () {
      return realtimeChannel.connect(realtimeChannel.socketUrl);
    });
  },


  /**
    * Disconnect connection
    * @memberof Board.RealtimeService
    * @param   {Board~Channel} channel
    * @returns {Promise}
    */
  disconnectMercuryConnection: function disconnectMercuryConnection(channel) {
    var _this4 = this;

    var realtimeChannel = this.realtimeChannels.get(channel.channelId);

    if (!realtimeChannel) {
      return _promise2.default.reject(new Error('Realtime Channel not found!'));
    }

    return realtimeChannel.disconnect()
    // even if we can't remove the channels from the collection, we can still
    // continue on execution
    .then((0, _common.tap)(function () {
      return _this4.realtimeChannels.remove(channel.channelId);
    }));
  },


  /**
   * Ensure board channelId is compatible with mercury bindings by replacing
   * '-' with '.' and '_' with '#'
   * @memberof Board.BoardService
   * @param   {String} channelId channel.channelId
   * @returns {String} mercury-binding compatible string
   */
  _boardChannelIdToMercuryBinding: function _boardChannelIdToMercuryBinding(channelId) {
    // make channelId mercury compatible replace `-` with `.` and `_` with `#`
    return this.config.mercuryBindingPrefix + channelId.replace(/-/g, '.').replace(/_/g, '#');
  },


  /**
   * Connect and use an exisiting mercury connection
   * @memberof Board.RealtimeService
   * @param  {Board~Channel} channel
   * @returns {Promise<Board~Registration>}
   */
  connectToSharedMercury: function connectToSharedMercury(channel) {
    var _this5 = this;

    return this.spark.internal.board.registerToShareMercury(channel).then(function (res) {
      _this5.realtimeChannels.add({
        channelId: channel.channelId,
        binding: res.binding,
        socketUrl: res.webSocketUrl
      });

      var realtimeChannel = _this5.realtimeChannels.get(channel.channelId);

      if (!res.sharedWebSocket) {
        return realtimeChannel.connect(realtimeChannel.socketUrl).then(function () {
          return res;
        });
      }

      realtimeChannel.isSharingMercury = true;
      realtimeChannel.socket = _this5.spark.internal.mercury.socket;
      // refresh socket reference when mercury is reconnected
      _this5.spark.internal.mercury.off('online', _this5.refreshMercurySocketReference, _this5);
      _this5.spark.internal.mercury.on('online', _this5.refreshMercurySocketReference, _this5);

      // make sure there's only one handler
      _this5.spark.internal.mercury.off('event:board.activity', _this5.handleBoardActivityMessages, _this5);
      _this5.spark.internal.mercury.on('event:board.activity', _this5.handleBoardActivityMessages, _this5);

      return res;
    });
  },
  handleBoardActivityMessages: function handleBoardActivityMessages(event) {
    var realtimeChannel = this.realtimeChannels.get(event.data.envelope.channelId);

    if (realtimeChannel) {
      realtimeChannel._emit('event:board.activity', event);
    }
  },
  refreshMercurySocketReference: function refreshMercurySocketReference() {
    var _this6 = this;

    this.realtimeChannels.forEach(function (realtimeChannel) {
      realtimeChannel.socket = _this6.spark.internal.mercury.socket;
    });
  },


  /**
   * Remove board binding from existing mercury connection
   * @memberof Board.RealtimeService
   * @param  {Board~Channel} channel
   * @returns {Promise<Board~Registration>}
   */
  disconnectFromSharedMercury: function disconnectFromSharedMercury(channel) {
    var _this7 = this;

    var realtimeChannel = this.realtimeChannels.get(channel.channelId);

    if (!realtimeChannel.isSharingMercury && realtimeChannel.socket && realtimeChannel.connected) {
      return this.disconnectMercuryConnection(channel);
    }

    return this.spark.internal.board.unregisterFromSharedMercury(channel, realtimeChannel.binding)
    // tap suppress errors but we can still go on if the channel can't be
    // removed from the collection
    .then((0, _common.tap)(function () {
      return _this7.realtimeChannels.remove(channel.channelId);
    }));
  },
  version: '1.52.7'
});

exports.default = RealtimeService;
//# sourceMappingURL=realtime.js.map
